#light_clock_pwm.py

# Same as light_clock_simple.py but following changes
#  - Using PWM for hour LEDs so can control brightness
#  - Addition of minutes at a different brightness to hour

import RPi.GPIO as GPIO
import time
import datetime
import schedule

alarmTime = '08:30'
alarmOff = '08:40'

# GPIO setup
GPIO.setmode(GPIO.BCM)      # Use the BCM pinout
GPIO.setwarnings(False)     # Ignore warnings about pins being used for other stuff
ledStripPin = 19            # LED strip is driven from this pin
hourPin = [25, 24, 23, 18, 15, 14, 8, 20, 16, 12, 1, 7] # REPLACE: the hour LEDs are connected to these pins (in order from 1 to 12)

# Set all the hourPins as outputs
for i in range(0, 12):
    GPIO.setup(hourPin[i], GPIO.OUT)
    GPIO.output(hourPin[i], 0)

# Start PWM on hour pins
hourPWM = []
for i in range(0, 12):
    p = GPIO.PWM(hourPin[i], 100)
    hourPWM.append(p)
    dc = 0
    p.start(dc)

GPIO.setup(ledStripPin, GPIO.OUT)       # Set ledStripPin as an output
pwm = GPIO.PWM(ledStripPin, 100)        # PWM on ledStripPin with frequency 100Hz
dutyCycle = 0                           # Initial brightness as percent
pwm.start(dutyCycle)

# Function declarations
def light_on():                         # Fade light up
    for dutyCycle in range(0, 101, 1):
        pwm.ChangeDutyCycle(dutyCycle)
        time.sleep(0.03)

def light_off():                        # Fade light down
    for dutyCycle in range(95, -1, -1):
        pwm.ChangeDutyCycle(dutyCycle)
        time.sleep(0.03)

def update_leds(h,m):                   # Update the LEDs to display h hours and m minutes
    # Turn off previous LED
    hourPWM[m-2].ChangeDutyCycle(0)
    hourPWM[h-2].ChangeDutyCycle(0)
    # Turn on new LED
    hourPWM[m-1].ChangeDutyCycle(20)    # 20% duty cycle
    hourPWM[h-1].ChangeDutyCycle(80)    # 80% duty cycle

schedule.every().day.at(alarmTime).do(light_on)     # At alarmTime turn the light on
schedule.every().day.at(alarmOff).do(light_off)     # At alarmOff turn the light off
# The above could be used to change the LED to various brighnesses through the day

lastHour = 0
lastMins = 0

try:
    while True:
        schedule.run_pending()          # Check for the alarm
        now = datetime.datetime.now()   # Get time
        hour = now.hour                 # Extract hours
        mins = now.minute               # Extract minutes
        if hour > 11:                   # Convert 24hr to 12hr clock
            hour = hour - 12
        mins = mins/5                   # Minute's corresponding hour location
        if mins != lastMins:            # Only update if necessary. This will
            update_leds(hour, mins)     # happen every 5 minutes.
        lastHour = hour
        lastMins = mins
        time.sleep(1)                   # Wait 1 second

except KeyboardInterrupt:               # If CTRL-C is pressed quit the program, but first:
    pwm.stop()                          # Stop the PWM
    for i in range(0, 12):
        hourPWM[i].stop()
    GPIO.cleanup()                      # Good practice before closing down